@@ -5,7 +5,6 @@ use std::{
     convert::Infallible,
     fmt::{Debug, Display},
     fs,
-    marker::PhantomData,
     ops::RangeBounds,
     path::{Path, PathBuf},
     sync::{
@@ -21,7 +20,7 @@ use parking_lot::Mutex;
 use crate::{
     context::Context,
     error::Error,
-    io::{FileManager, ManagedFile},
+    io::{fs::StdFileManager, FileManager, ManagedFile},
     transaction::{LogEntry, TransactionHandle, TransactionManager},
     tree::{
         self, root::AnyTreeRoot, state::AnyTreeState, EmbeddedIndex, KeyEvaluation, KeySequence,
@@ -412,7 +411,7 @@ impl<Root: tree::Root, File: ManagedFile> TransactionTree<Root, File> {
     pub fn compare_and_swap(
         &mut self,
         key: &[u8],
-        old: Option<&ArcBytes<'_>>,
+        old: Option<&[u8]>,
         new: Option<ArcBytes<'_>>,
     ) -> Result<(), CompareAndSwapError> {
         self.tree
@@ -478,7 +477,19 @@ impl<Root: tree::Root, File: ManagedFile> TransactionTree<Root, File> {
         )
     }
 
-    /// Returns the last  of the tree.
+    /// Returns the first key of the tree.
+    #[cfg_attr(feature = "tracing", tracing::instrument(skip(self)))]
+    pub fn first_key(&mut self) -> Result<Option<ArcBytes<'static>>, Error> {
+        self.tree.first_key(true)
+    }
+
+    /// Returns the first key and value of the tree.
+    #[cfg_attr(feature = "tracing", tracing::instrument(skip(self)))]
+    pub fn first(&mut self) -> Result<Option<(ArcBytes<'static>, ArcBytes<'static>)>, Error> {
+        self.tree.first(true)
+    }
+
+    /// Returns the last key of the tree.
     #[cfg_attr(feature = "tracing", tracing::instrument(skip(self)))]
     pub fn last_key(&mut self) -> Result<Option<ArcBytes<'static>>, Error> {
         self.tree.last_key(true)
@@ -505,16 +516,27 @@ pub enum CompareAndSwapError {
 /// A database configuration used to open a database.
 #[derive(Debug)]
 #[must_use]
-pub struct Config<File: ManagedFile> {
+pub struct Config<M: FileManager = StdFileManager> {
     path: PathBuf,
     vault: Option<Arc<dyn AnyVault>>,
     cache: Option<ChunkCache>,
-    file_manager: Option<File::Manager>,
-    thread_pool: Option<ThreadPool<File>>,
-    _file: PhantomData<File>,
+    file_manager: Option<M>,
+    thread_pool: Option<ThreadPool<M::File>>,
 }
 
-impl<File: ManagedFile> Config<File> {
+impl<M: FileManager> Clone for Config<M> {
+    fn clone(&self) -> Self {
+        Self {
+            path: self.path.clone(),
+            vault: self.vault.clone(),
+            cache: self.cache.clone(),
+            file_manager: self.file_manager.clone(),
+            thread_pool: self.thread_pool.clone(),
+        }
+    }
+}
+
+impl Config<StdFileManager> {
     /// Creates a new config to open a database located at `path`.
     pub fn new<P: AsRef<Path>>(path: P) -> Self {
         Self {
@@ -523,9 +545,9 @@ impl<File: ManagedFile> Config<File> {
             cache: None,
             thread_pool: None,
             file_manager: None,
-            _file: PhantomData,
         }
     }
+
     /// Returns a default configuration to open a database located at `path`.
     pub fn default_for<P: AsRef<Path>>(path: P) -> Self {
         Self {
@@ -534,10 +556,27 @@ impl<File: ManagedFile> Config<File> {
             cache: Some(ChunkCache::new(2000, 65536)),
             thread_pool: Some(ThreadPool::default()),
             file_manager: None,
-            _file: PhantomData,
         }
     }
 
+    /// Sets the file manager.
+    ///
+    /// ## Panics
+    ///
+    /// Panics if called after a shared thread pool has been set.
+    pub fn file_manager<M: FileManager>(self, file_manager: M) -> Config<M> {
+        assert!(self.thread_pool.is_none());
+        Config {
+            path: self.path,
+            vault: self.vault,
+            cache: self.cache,
+            file_manager: Some(file_manager),
+            thread_pool: None,
+        }
+    }
+}
+
+impl<M: FileManager> Config<M> {
     /// Sets the vault to use for this database.
     pub fn vault<V: AnyVault>(mut self, vault: V) -> Self {
         self.vault = Some(Arc::new(vault));
@@ -550,22 +589,16 @@ impl<File: ManagedFile> Config<File> {
         self
     }
 
-    /// Sets the file manager.
-    pub fn file_manager(mut self, file_manager: File::Manager) -> Self {
-        self.file_manager = Some(file_manager);
-        self
-    }
-
     /// Uses the `thread_pool` provided instead of creating its own. This will
     /// allow a single thread pool to manage multiple [`Roots`] instances'
     /// transactions.
-    pub fn shared_thread_pool(mut self, thread_pool: &ThreadPool<File>) -> Self {
+    pub fn shared_thread_pool(mut self, thread_pool: &ThreadPool<M::File>) -> Self {
         self.thread_pool = Some(thread_pool.clone());
         self
     }
 
     /// Opens the database, or creates one if the target path doesn't exist.
-    pub fn open(self) -> Result<Roots<File>, Error> {
+    pub fn open(self) -> Result<Roots<M::File>, Error> {
         Roots::open(
             self.path,
             Context {
@@ -706,7 +739,7 @@ impl<Root: tree::Root, File: ManagedFile> Tree<Root, File> {
     pub fn compare_and_swap(
         &self,
         key: &[u8],
-        old: Option<&ArcBytes<'_>>,
+        old: Option<&[u8]>,
         new: Option<ArcBytes<'_>>,
     ) -> Result<(), CompareAndSwapError> {
         let mut transaction = self.begin_transaction()?;
@@ -789,6 +822,26 @@ impl<Root: tree::Root, File: ManagedFile> Tree<Root, File> {
         })
     }
 
+    /// Returns the first key of the tree.
+    #[cfg_attr(feature = "tracing", tracing::instrument(skip(self)))]
+    pub fn first_key(&self) -> Result<Option<ArcBytes<'static>>, Error> {
+        catch_compaction_and_retry(|| {
+            let mut tree = self.open_for_read()?;
+
+            tree.first_key(false)
+        })
+    }
+
+    /// Returns the first key and value of the tree.
+    #[cfg_attr(feature = "tracing", tracing::instrument(skip(self)))]
+    pub fn first(&self) -> Result<Option<(ArcBytes<'static>, ArcBytes<'static>)>, Error> {
+        catch_compaction_and_retry(|| {
+            let mut tree = self.open_for_read()?;
+
+            tree.first(false)
+        })
+    }
+
     /// Returns the last key of the tree.
     #[cfg_attr(feature = "tracing", tracing::instrument(skip(self)))]
     pub fn last_key(&self) -> Result<Option<ArcBytes<'static>>, Error> {
@@ -1088,14 +1141,17 @@ mod tests {
 
     use super::*;
     use crate::{
-        io::{fs::StdFile, memory::MemoryFile, ManagedFile},
+        io::{any::AnyFileManager, fs::StdFileManager, memory::MemoryFileManager},
         test_util::RotatorVault,
         tree::{Root, Unversioned, Versioned},
     };
 
-    fn basic_get_set<File: ManagedFile>() {
+    fn basic_get_set<M: FileManager>(file_manager: M) {
         let tempdir = tempdir().unwrap();
-        let roots = Config::<File>::new(tempdir.path()).open().unwrap();
+        let roots = Config::new(tempdir.path())
+            .file_manager(file_manager)
+            .open()
+            .unwrap();
 
         let tree = roots.tree(Versioned::tree("test")).unwrap();
         tree.set(b"test", b"value").unwrap();
@@ -1106,19 +1162,21 @@ mod tests {
 
     #[test]
     fn memory_basic_get_set() {
-        basic_get_set::<MemoryFile>();
+        basic_get_set(MemoryFileManager::default());
     }
 
     #[test]
     fn std_basic_get_set() {
-        basic_get_set::<StdFile>();
+        basic_get_set(StdFileManager::default());
     }
 
     #[test]
     fn basic_transaction_isolation_test() {
         let tempdir = tempdir().unwrap();
 
-        let roots = Config::<StdFile>::new(tempdir.path()).open().unwrap();
+        let roots = Config::<StdFileManager>::new(tempdir.path())
+            .open()
+            .unwrap();
         let tree = roots.tree(Versioned::tree("test")).unwrap();
         tree.set(b"test", b"value").unwrap();
 
@@ -1157,7 +1215,9 @@ mod tests {
     fn basic_transaction_rollback_test() {
         let tempdir = tempdir().unwrap();
 
-        let roots = Config::<StdFile>::new(tempdir.path()).open().unwrap();
+        let roots = Config::<StdFileManager>::new(tempdir.path())
+            .open()
+            .unwrap();
         let tree = roots.tree(Versioned::tree("test")).unwrap();
         tree.set(b"test", b"value").unwrap();
 
@@ -1191,21 +1251,46 @@ mod tests {
     }
 
     #[test]
-    fn compact_test_versioned() {
-        compact_test::<Versioned>();
+    fn std_compact_test_versioned() {
+        compact_test::<Versioned, _>(StdFileManager::default());
     }
 
     #[test]
-    fn compact_test_unversioned() {
-        compact_test::<Unversioned>();
+    fn std_compact_test_unversioned() {
+        compact_test::<Unversioned, _>(StdFileManager::default());
     }
 
-    fn compact_test<R: Root>() {
+    #[test]
+    fn memory_compact_test_versioned() {
+        compact_test::<Versioned, _>(MemoryFileManager::default());
+    }
+
+    #[test]
+    fn memory_compact_test_unversioned() {
+        compact_test::<Unversioned, _>(MemoryFileManager::default());
+    }
+
+    #[test]
+    fn any_compact_test_versioned() {
+        compact_test::<Versioned, _>(AnyFileManager::std());
+        compact_test::<Versioned, _>(AnyFileManager::memory());
+    }
+
+    #[test]
+    fn any_compact_test_unversioned() {
+        compact_test::<Unversioned, _>(AnyFileManager::std());
+        compact_test::<Unversioned, _>(AnyFileManager::memory());
+    }
+
+    fn compact_test<R: Root, M: FileManager>(file_manager: M) {
         const OPERATION_COUNT: usize = 256;
         const WORKER_COUNT: usize = 4;
         let tempdir = tempdir().unwrap();
 
-        let roots = Config::<StdFile>::new(tempdir.path()).open().unwrap();
+        let roots = Config::new(tempdir.path())
+            .file_manager(file_manager)
+            .open()
+            .unwrap();
         let tree = roots.tree(R::tree("test")).unwrap();
         tree.set("foo", b"bar").unwrap();
 
@@ -1261,7 +1346,7 @@ mod tests {
 
         // Encrypt a tree using default encryption via the context
         {
-            let roots = Config::<StdFile>::new(tempdir.path())
+            let roots = Config::<StdFileManager>::new(tempdir.path())
                 .vault(RotatorVault::new(13))
                 .open()
                 .unwrap();
@@ -1274,7 +1359,7 @@ mod tests {
         }
         // Try to access the tree with the vault again.
         {
-            let roots = Config::<StdFile>::new(tempdir.path())
+            let roots = Config::<StdFileManager>::new(tempdir.path())
                 .vault(RotatorVault::new(13))
                 .open()
                 .unwrap();
@@ -1294,7 +1379,9 @@ mod tests {
             assert_eq!(value.as_deref(), Some(&b"other"[..]));
         }
         {
-            let roots = Config::<StdFile>::new(tempdir.path()).open().unwrap();
+            let roots = Config::<StdFileManager>::new(tempdir.path())
+                .open()
+                .unwrap();
             // Try to access roots without the vault.
             let bad_tree = roots.tree(Versioned::tree("test")).unwrap();
             assert!(bad_tree.get(b"test").is_err());
@@ -1306,4 +1393,50 @@ mod tests {
             assert_eq!(bad_tree.get(b"test").unwrap(), None);
         }
     }
+
+    #[test]
+    fn too_large_transaction() {
+        let tempdir = tempdir().unwrap();
+
+        let config = Config::<StdFileManager>::new(tempdir.path());
+        {
+            let roots = config.clone().open().unwrap();
+
+            let mut transaction = roots.transaction(&[Versioned::tree("test")]).unwrap();
+
+            // Write some data to the tree.
+            transaction
+                .tree::<Versioned>(0)
+                .unwrap()
+                .set(b"test", vec![0; 16 * 1024 * 1024])
+                .unwrap();
+
+            // Issue a transaction that's too large.
+            assert!(matches!(
+                transaction
+                    .entry_mut()
+                    .set_data(vec![0; 16 * 1024 * 1024 - 7])
+                    .unwrap_err()
+                    .kind,
+                ErrorKind::ValueTooLarge
+            ));
+            // Roll the transaction back
+            drop(transaction);
+        }
+        // Ensure that we can still write to the tree.
+        {
+            let roots = config.open().unwrap();
+
+            let mut transaction = roots.transaction(&[Versioned::tree("test")]).unwrap();
+
+            // Write some data to the tree
+            transaction
+                .tree::<Versioned>(0)
+                .unwrap()
+                .set(b"test", b"updated value")
+                .unwrap();
+
+            transaction.commit().unwrap();
+        }
+    }
 }
