@@ -45,7 +45,7 @@ use std::{
     io::SeekFrom,
     marker::PhantomData,
     ops::{Bound, Deref, DerefMut, Range, RangeBounds},
-    path::{Path, PathBuf},
+    path::Path,
     sync::Arc,
 };
 
@@ -56,7 +56,7 @@ use parking_lot::MutexGuard;
 use crate::{
     chunk_cache::CacheEntry,
     error::Error,
-    io::{FileManager, FileOp, ManagedFile, OpenableFile},
+    io::{File, FileManager, FileOp, ManagedFile, ManagedFileOpener, OpenableFile, OperableFile},
     roots::AbortError,
     transaction::{TransactionHandle, TransactionManager},
     tree::{btree_entry::ScanArgs, serialization::BinarySerialization},
@@ -221,7 +221,7 @@ impl<Root: root::Root, File: ManagedFile> TreeFile<Root, File> {
             return Ok(());
         }
 
-        let mut tree = File::open_for_read(file_path, None)?;
+        let mut tree = context.file_manager.open_for_read(file_path, None)?;
 
         // Scan back block by block until we find a header page.
         let mut block_start = file_length - (file_length % PAGE_SIZE as u64);
@@ -295,12 +295,14 @@ impl<Root: root::Root, File: ManagedFile> TreeFile<Root, File> {
         Ok(())
     }
 
-    /// Pushes a key/value pair. Replaces any previous value if set.
-    pub fn push(
+    /// Sets a key/value pair. Replaces any previous value if set. If you wish
+    /// to retrieve the previously stored value, use
+    /// [`replace()`](Self::replace) instead.
+    pub fn set(
         &mut self,
         transaction_id: Option<u64>,
-        key: ArcBytes<'static>,
-        value: ArcBytes<'static>,
+        key: impl Into<ArcBytes<'static>>,
+        value: impl Into<ArcBytes<'static>>,
     ) -> Result<(), Error> {
         self.file.execute(TreeModifier {
             state: &self.state,
@@ -308,8 +310,8 @@ impl<Root: root::Root, File: ManagedFile> TreeFile<Root, File> {
             cache: self.cache.as_ref(),
             modification: Some(Modification {
                 transaction_id,
-                keys: vec![key],
-                operation: Operation::Set(value),
+                keys: vec![key.into()],
+                operation: Operation::Set(value.into()),
             }),
             scratch: &mut self.scratch,
         })
@@ -335,7 +337,7 @@ impl<Root: root::Root, File: ManagedFile> TreeFile<Root, File> {
     pub fn compare_and_swap(
         &mut self,
         key: &[u8],
-        old: Option<&ArcBytes<'_>>,
+        old: Option<&[u8]>,
         mut new: Option<ArcBytes<'_>>,
         transaction_id: Option<u64>,
     ) -> Result<(), CompareAndSwapError> {
@@ -343,8 +345,11 @@ impl<Root: root::Root, File: ManagedFile> TreeFile<Root, File> {
         self.modify(Modification {
             transaction_id,
             keys: vec![ArcBytes::from(key)],
-            operation: Operation::CompareSwap(CompareSwap::new(&mut |_key, value| {
-                if value.as_ref() == old {
+            operation: Operation::CompareSwap(CompareSwap::new(&mut |_key,
+                                                                     value: Option<
+                ArcBytes<'_>,
+            >| {
+                if value.as_deref() == old {
                     match new.take() {
                         Some(new) => KeyOperation::Set(new.into_owned()),
                         None => KeyOperation::Remove,
@@ -519,6 +524,53 @@ impl<Root: root::Root, File: ManagedFile> TreeFile<Root, File> {
         Ok(())
     }
 
+    /// Returns the last key of the tree.
+    pub fn first_key(&mut self, in_transaction: bool) -> Result<Option<ArcBytes<'static>>, Error> {
+        let mut result = None;
+        self.scan(
+            &(..),
+            true,
+            in_transaction,
+            &mut |_, _, _| true,
+            &mut |key, _index| {
+                result = Some(key.clone());
+                KeyEvaluation::Stop
+            },
+            &mut |_key, _index, _value| Ok(()),
+        )?;
+
+        Ok(result)
+    }
+
+    /// Returns the first key and value of the tree.
+    pub fn first(
+        &mut self,
+        in_transaction: bool,
+    ) -> Result<Option<(ArcBytes<'static>, ArcBytes<'static>)>, Error> {
+        let mut result = None;
+        let mut key_requested = false;
+        self.scan(
+            &(..),
+            true,
+            in_transaction,
+            &mut |_, _, _| true,
+            &mut |_, _| {
+                if key_requested {
+                    KeyEvaluation::Stop
+                } else {
+                    key_requested = true;
+                    KeyEvaluation::ReadData
+                }
+            },
+            &mut |key, _index, value| {
+                result = Some((key, value));
+                Ok(())
+            },
+        )?;
+
+        Ok(result)
+    }
+
     /// Returns the last key of the tree.
     pub fn last_key(&mut self, in_transaction: bool) -> Result<Option<ArcBytes<'static>>, Error> {
         let mut result = None;
@@ -589,15 +641,16 @@ impl<Root: root::Root, File: ManagedFile> TreeFile<Root, File> {
         file_manager: &File::Manager,
         transactions: Option<TransactableCompaction<'_, File::Manager>>,
     ) -> Result<Self, Error> {
-        let (compacted_path, finisher) = self.file.execute(TreeCompactor {
+        let (compacted_file, finisher) = self.file.execute(TreeCompactor {
             state: &self.state,
+            manager: file_manager,
             vault: self.vault.as_deref(),
             transactions,
             scratch: &mut self.scratch,
         })?;
         self.file = self
             .file
-            .replace_with(&compacted_path, file_manager, |file_id| {
+            .replace_with(compacted_file, file_manager, |file_id| {
                 finisher.finish(file_id);
             })?;
         Ok(self)
@@ -663,17 +716,22 @@ pub struct TransactableCompaction<'a, Manager: FileManager> {
 }
 
 struct TreeCompactor<'a, Root: root::Root, Manager: FileManager> {
+    manager: &'a Manager,
     state: &'a State<Root>,
     vault: Option<&'a dyn AnyVault>,
     transactions: Option<TransactableCompaction<'a, Manager>>,
     scratch: &'a mut Vec<u8>,
 }
-impl<'a, Root: root::Root, File: ManagedFile> FileOp<File>
-    for TreeCompactor<'a, Root, File::Manager>
+impl<'a, Root, Manager> FileOp<Result<(Manager::File, TreeCompactionFinisher<'a, Root>), Error>>
+    for TreeCompactor<'a, Root, Manager>
+where
+    Root: root::Root,
+    Manager: FileManager,
 {
-    type Output = Result<(PathBuf, TreeCompactionFinisher<'a, Root>), Error>;
-
-    fn execute(self, file: &mut File) -> Self::Output {
+    fn execute(
+        self,
+        file: &mut dyn File,
+    ) -> Result<(Manager::File, TreeCompactionFinisher<'a, Root>), Error> {
         let current_path = file.path().to_path_buf();
         let file_name = current_path
             .file_name()
@@ -694,7 +752,7 @@ impl<'a, Root: root::Root, File: ManagedFile> FileOp<File>
                 .manager
                 .new_transaction([transactions.name.as_bytes()])
         });
-        let mut new_file = File::open_for_append(&compacted_path, None)?;
+        let mut new_file = self.manager.open_for_append(&compacted_path, None)?;
         let mut writer = PagedWriter::new(None, &mut new_file, self.vault, None, 0)?;
 
         // Use the read state to list all the currently live chunks
@@ -718,7 +776,7 @@ impl<'a, Root: root::Root, File: ManagedFile> FileOp<File>
         // file.
 
         Ok((
-            compacted_path,
+            new_file,
             TreeCompactionFinisher {
                 write_state,
                 state: self.state,
@@ -749,9 +807,11 @@ struct TreeWriter<'a, Root: root::Root> {
     scratch: &'a mut Vec<u8>,
 }
 
-impl<'a, Root: root::Root, File: ManagedFile> FileOp<File> for TreeWriter<'a, Root> {
-    type Output = Result<(), Error>;
-    fn execute(self, file: &mut File) -> Self::Output {
+impl<'a, Root> FileOp<Result<(), Error>> for TreeWriter<'a, Root>
+where
+    Root: root::Root,
+{
+    fn execute(self, file: &mut dyn File) -> Result<(), Error> {
         let mut active_state = self.state.lock();
         if active_state.file_id != file.id() {
             return Err(Error::from(ErrorKind::TreeCompacted));
@@ -787,10 +847,11 @@ struct TreeModifier<'a, 'm, Root: root::Root> {
     scratch: &'a mut Vec<u8>,
 }
 
-impl<'a, 'm, Root: root::Root, File: ManagedFile> FileOp<File> for TreeModifier<'a, 'm, Root> {
-    type Output = Result<(), Error>;
-
-    fn execute(mut self, file: &mut File) -> Self::Output {
+impl<'a, 'm, Root> FileOp<Result<(), Error>> for TreeModifier<'a, 'm, Root>
+where
+    Root: root::Root,
+{
+    fn execute(mut self, file: &mut dyn File) -> Result<(), Error> {
         let mut active_state = self.state.lock();
         if active_state.file_id != file.id() {
             return Err(Error::from(ErrorKind::TreeCompacted));
@@ -835,11 +896,11 @@ impl<'a, 'm, Root: root::Root, File: ManagedFile> FileOp<File> for TreeModifier<
 }
 
 #[allow(clippy::shadow_unrelated)] // It is related, but clippy can't tell.
-fn save_tree<Root: root::Root, File: ManagedFile>(
+fn save_tree<Root: root::Root>(
     active_state: &mut ActiveState<Root>,
     vault: Option<&dyn AnyVault>,
     cache: Option<&ChunkCache>,
-    mut data_block: PagedWriter<'_, File>,
+    mut data_block: PagedWriter<'_>,
     scratch: &mut Vec<u8>,
 ) -> Result<(), Error> {
     scratch.clear();
@@ -921,17 +982,15 @@ struct TreeGetter<
     key_reader: KeyReader,
 }
 
-impl<'a, 'keys, KeyEvaluator, KeyReader, Root, File, Keys> FileOp<File>
+impl<'a, 'keys, KeyEvaluator, KeyReader, Root, Keys> FileOp<Result<(), Error>>
     for TreeGetter<'a, 'keys, Root, KeyEvaluator, KeyReader, Keys>
 where
     KeyEvaluator: FnMut(&ArcBytes<'static>) -> KeyEvaluation,
     KeyReader: FnMut(ArcBytes<'static>, ArcBytes<'static>) -> Result<(), Error>,
     Keys: Iterator<Item = &'keys [u8]>,
     Root: root::Root,
-    File: ManagedFile,
 {
-    type Output = Result<(), Error>;
-    fn execute(mut self, file: &mut File) -> Self::Output {
+    fn execute(mut self, file: &mut dyn File) -> Result<(), Error> {
         if self.from_transaction {
             let state = self.state.lock();
             if state.file_id != file.id() {
@@ -1005,8 +1064,7 @@ impl<
         KeyEvaluator,
         KeyReader,
         KeyRangeBounds,
-        File,
-    > FileOp<File>
+    > FileOp<Result<bool, AbortError<CallerError>>>
     for TreeScanner<
         'a,
         'keys,
@@ -1018,7 +1076,6 @@ impl<
         KeyRangeBounds,
     >
 where
-    File: ManagedFile,
     NodeEvaluator: FnMut(&ArcBytes<'static>, &Root::ReducedIndex, usize) -> bool,
     KeyEvaluator: FnMut(&ArcBytes<'static>, &Root::Index) -> KeyEvaluation,
     KeyReader: FnMut(
@@ -1029,8 +1086,7 @@ where
     KeyRangeBounds: RangeBounds<&'keys [u8]> + Debug + ?Sized,
     CallerError: Display + Debug,
 {
-    type Output = Result<bool, AbortError<CallerError>>;
-    fn execute(mut self, file: &mut File) -> Self::Output {
+    fn execute(mut self, file: &mut dyn File) -> Result<bool, AbortError<CallerError>> {
         if self.from_transaction {
             let state = self.state.lock();
             if state.file_id != file.id() {
@@ -1095,7 +1151,8 @@ struct TreeSequenceScanner<
     data_callback: DataCallback,
 }
 
-impl<'a, 'keys, KeyEvaluator, KeyRangeBounds, File, DataCallback, CallerError, Index> FileOp<File>
+impl<'a, 'keys, KeyEvaluator, KeyRangeBounds, DataCallback, CallerError, Index>
+    FileOp<Result<(), AbortError<CallerError>>>
     for TreeSequenceScanner<
         'a,
         'keys,
@@ -1106,15 +1163,13 @@ impl<'a, 'keys, KeyEvaluator, KeyRangeBounds, File, DataCallback, CallerError, I
         Index,
     >
 where
-    File: ManagedFile,
     KeyEvaluator: FnMut(&ArcBytes<'static>, &BySequenceIndex) -> KeyEvaluation,
     KeyRangeBounds: RangeBounds<&'keys [u8]> + Debug + ?Sized,
     DataCallback: FnMut(KeySequence, ArcBytes<'static>) -> Result<(), AbortError<CallerError>>,
     CallerError: Display + Debug,
     Index: EmbeddedIndex + Reducer<Index> + Clone + Debug + 'static,
 {
-    type Output = Result<(), AbortError<CallerError>>;
-    fn execute(self, file: &mut File) -> Self::Output {
+    fn execute(self, file: &mut dyn File) -> Result<(), AbortError<CallerError>> {
         let Self {
             forwards,
             from_transaction,
@@ -1189,8 +1244,8 @@ where
 }
 
 /// Writes data in pages, allowing for quick scanning through the file.
-pub struct PagedWriter<'a, File: ManagedFile> {
-    file: &'a mut File,
+pub struct PagedWriter<'a> {
+    file: &'a mut dyn File,
     vault: Option<&'a dyn AnyVault>,
     cache: Option<&'a ChunkCache>,
     position: u64,
@@ -1198,15 +1253,15 @@ pub struct PagedWriter<'a, File: ManagedFile> {
     buffered_write: [u8; WRITE_BUFFER_SIZE],
 }
 
-impl<'a, File: ManagedFile> Deref for PagedWriter<'a, File> {
-    type Target = File;
+impl<'a> Deref for PagedWriter<'a> {
+    type Target = dyn File;
 
     fn deref(&self) -> &Self::Target {
         self.file
     }
 }
 
-impl<'a, File: ManagedFile> DerefMut for PagedWriter<'a, File> {
+impl<'a> DerefMut for PagedWriter<'a> {
     fn deref_mut(&mut self) -> &mut Self::Target {
         self.file
     }
@@ -1214,14 +1269,14 @@ impl<'a, File: ManagedFile> DerefMut for PagedWriter<'a, File> {
 
 const WRITE_BUFFER_SIZE: usize = 8 * 1024;
 
-impl<'a, File: ManagedFile> PagedWriter<'a, File> {
+impl<'a> PagedWriter<'a> {
     fn new(
         header: Option<PageHeader>,
-        file: &'a mut File,
+        file: &'a mut dyn File,
         vault: Option<&'a dyn AnyVault>,
         cache: Option<&'a ChunkCache>,
         position: u64,
-    ) -> Result<PagedWriter<'a, File>, Error> {
+    ) -> Result<Self, Error> {
         let mut writer = Self {
             file,
             vault,
@@ -1249,7 +1304,7 @@ impl<'a, File: ManagedFile> PagedWriter<'a, File> {
         Ok(writer)
     }
 
-    fn current_position(&self) -> u64 {
+    const fn current_position(&self) -> u64 {
         self.position + self.offset as u64
     }
 
@@ -1329,7 +1384,7 @@ impl<'a, File: ManagedFile> PagedWriter<'a, File> {
         self.write(&buffer)
     }
 
-    fn finish(mut self) -> Result<(&'a mut File, u64), Error> {
+    fn finish(mut self) -> Result<(&'a mut dyn File, u64), Error> {
         self.commit_if_needed()?;
         Ok((self.file, self.position))
     }
@@ -1337,10 +1392,10 @@ impl<'a, File: ManagedFile> PagedWriter<'a, File> {
 
 #[allow(clippy::cast_possible_truncation)]
 #[cfg_attr(feature = "tracing", tracing::instrument(skip(file, vault, cache)))]
-fn read_chunk<File: ManagedFile>(
+fn read_chunk(
     position: u64,
     validate_crc: bool,
-    file: &mut File,
+    file: &mut dyn File,
     vault: Option<&dyn AnyVault>,
     cache: Option<&ChunkCache>,
 ) -> Result<CacheEntry, Error> {
@@ -1383,11 +1438,11 @@ fn read_chunk<File: ManagedFile>(
     Ok(CacheEntry::ArcBytes(decrypted))
 }
 
-pub(crate) fn copy_chunk<File: ManagedFile, Hasher: BuildHasher>(
+pub(crate) fn copy_chunk<Hasher: BuildHasher>(
     original_position: u64,
-    from_file: &mut File,
+    from_file: &mut dyn File,
     copied_chunks: &mut std::collections::HashMap<u64, u64, Hasher>,
-    to_file: &mut PagedWriter<'_, File>,
+    to_file: &mut PagedWriter<'_>,
     vault: Option<&dyn AnyVault>,
 ) -> Result<u64, Error> {
     if original_position == 0 {
@@ -1578,12 +1633,15 @@ mod tests {
 
     use super::*;
     use crate::io::{
+        any::AnyFileManager,
         fs::{StdFile, StdFileManager},
-        memory::{MemoryFile, MemoryFileManager},
+        memory::{MemoryFile, MemoryFileManager, MemoryFileOpener},
+        ManagedFileOpener,
     };
 
     fn test_paged_write(offset: usize, length: usize) -> Result<(), Error> {
-        let mut file = MemoryFile::open_for_append(format!("test-{}-{}", offset, length), None)?;
+        let mut file =
+            MemoryFileOpener.open_for_append(format!("test-{}-{}", offset, length), None)?;
         let mut paged_writer =
             PagedWriter::new(Some(PageHeader::VersionedHeader), &mut file, None, None, 0)?;
 
@@ -1650,7 +1708,7 @@ mod tests {
             let mut tree =
                 TreeFile::<R, F>::write(file_path, State::new(None, max_order), context, None)
                     .unwrap();
-            tree.push(None, id_buffer.clone(), ArcBytes::from(b"hello world"))
+            tree.set(None, id_buffer.clone(), ArcBytes::from(b"hello world"))
                 .unwrap();
 
             // This shouldn't have to scan the file, as the data fits in memory.
@@ -1846,27 +1904,49 @@ mod tests {
         let mut tree = TreeFile::<R, F>::write(file_path, state, &context, None).unwrap();
         for (_index, id) in ids.enumerate() {
             let id_buffer = ArcBytes::from(id.to_be_bytes().to_vec());
-            tree.push(None, id_buffer.clone(), ArcBytes::from(b"hello world"))
+            tree.set(None, id_buffer.clone(), ArcBytes::from(b"hello world"))
                 .unwrap();
         }
     }
 
     #[test]
-    fn bulk_insert_versioned() {
-        bulk_insert::<Versioned, StdFile>("std-versioned");
+    fn std_bulk_insert_versioned() {
+        bulk_insert::<Versioned, _>("std-versioned", StdFileManager::default());
     }
 
     #[test]
-    fn bulk_insert_unversioned() {
-        bulk_insert::<Unversioned, StdFile>("std-unversioned");
+    fn memory_bulk_insert_versioned() {
+        bulk_insert::<Versioned, _>("memory-versioned", MemoryFileManager::default());
     }
 
-    fn bulk_insert<R: Root, F: ManagedFile>(name: &str) {
+    #[test]
+    fn any_bulk_insert_versioned() {
+        bulk_insert::<Versioned, _>("any-versioned", AnyFileManager::memory());
+        bulk_insert::<Versioned, _>("any-versioned", AnyFileManager::std());
+    }
+
+    #[test]
+    fn std_bulk_insert_unversioned() {
+        bulk_insert::<Unversioned, _>("std-unversioned", StdFileManager::default());
+    }
+
+    #[test]
+    fn memory_bulk_insert_unversioned() {
+        bulk_insert::<Unversioned, _>("memory-unversioned", MemoryFileManager::default());
+    }
+
+    #[test]
+    fn any_bulk_insert_unversioned() {
+        bulk_insert::<Unversioned, _>("any-unversioned", AnyFileManager::memory());
+        bulk_insert::<Unversioned, _>("any-unversioned", AnyFileManager::std());
+    }
+
+    fn bulk_insert<R: Root, M: FileManager>(name: &str, file_manager: M) {
         const RECORDS_PER_BATCH: usize = 10;
         const BATCHES: usize = 1000;
         let mut rng = Pcg64::new_seed(1);
         let context = Context {
-            file_manager: F::Manager::default(),
+            file_manager,
             vault: None,
             cache: Some(ChunkCache::new(100, 160_384)),
         };
@@ -1874,7 +1954,7 @@ mod tests {
         std::fs::create_dir(&temp_dir).unwrap();
         let file_path = temp_dir.join("tree");
         let state = State::default();
-        let mut tree = TreeFile::<R, F>::write(file_path, state, &context, None).unwrap();
+        let mut tree = TreeFile::<R, M::File>::write(file_path, state, &context, None).unwrap();
         for _ in 0..BATCHES {
             let mut ids = (0..RECORDS_PER_BATCH)
                 .map(|_| rng.generate::<u64>())
@@ -1915,7 +1995,7 @@ mod tests {
         let mut ids = Vec::new();
         for id in 0..3_u32.pow(4) {
             let id_buffer = ArcBytes::from(id.to_be_bytes().to_vec());
-            tree.push(None, id_buffer.clone(), id_buffer.clone())
+            tree.set(None, id_buffer.clone(), id_buffer.clone())
                 .unwrap();
             ids.push(id_buffer);
         }
@@ -1959,11 +2039,11 @@ mod tests {
         assert_eq!(&all_records, &all_through_scan);
     }
 
-    fn compact<R: Root>(label: &str) {
+    fn compact<R: Root, M: FileManager>(label: &str, file_manager: M) {
         const ORDER: usize = 4;
         let mut rng = Pcg64::new_seed(1);
         let context = Context {
-            file_manager: StdFileManager::default(),
+            file_manager,
             vault: None,
             cache: None,
         };
@@ -1972,11 +2052,11 @@ mod tests {
         let file_path = temp_dir.join("tree");
         let mut ids = HashSet::new();
         for _ in 0..5 {
-            insert_one_record::<R, StdFile>(&context, &file_path, &mut ids, &mut rng, Some(ORDER));
+            insert_one_record::<R, M::File>(&context, &file_path, &mut ids, &mut rng, Some(ORDER));
         }
 
         let mut tree =
-            TreeFile::<R, StdFile>::write(&file_path, State::default(), &context, None).unwrap();
+            TreeFile::<R, M::File>::write(&file_path, State::default(), &context, None).unwrap();
         let pre_compact_size = context.file_manager.file_length(&file_path).unwrap();
         tree = tree.compact(&context.file_manager, None).unwrap();
         let after_compact_size = context.file_manager.file_length(&file_path).unwrap();
@@ -1995,13 +2075,35 @@ mod tests {
     }
 
     #[test]
-    fn compact_versioned() {
-        compact::<Versioned>("versioned");
+    fn std_compact_versioned() {
+        compact::<Versioned, _>("versioned", StdFileManager::default());
     }
 
     #[test]
-    fn compact_unversioned() {
-        compact::<Unversioned>("unversioned");
+    fn std_compact_unversioned() {
+        compact::<Unversioned, _>("unversioned", StdFileManager::default());
+    }
+
+    #[test]
+    fn memory_compact_versioned() {
+        compact::<Versioned, _>("versioned", MemoryFileManager::default());
+    }
+
+    #[test]
+    fn memory_compact_unversioned() {
+        compact::<Unversioned, _>("unversioned", MemoryFileManager::default());
+    }
+
+    #[test]
+    fn any_compact_versioned() {
+        compact::<Versioned, _>("any-versioned", AnyFileManager::std());
+        compact::<Versioned, _>("any-versioned", AnyFileManager::memory());
+    }
+
+    #[test]
+    fn any_compact_unversioned() {
+        compact::<Unversioned, _>("any-unversioned", AnyFileManager::std());
+        compact::<Unversioned, _>("any-unversioned", AnyFileManager::memory());
     }
 
     #[test]
@@ -2017,11 +2119,11 @@ mod tests {
             TreeFile::<Versioned, StdFile>::write(tempfile.path(), state, &context, None).unwrap();
 
         // Store three versions of the same key.
-        tree.push(None, ArcBytes::from(b"a"), ArcBytes::from(b"0"))
+        tree.set(None, ArcBytes::from(b"a"), ArcBytes::from(b"0"))
             .unwrap();
-        tree.push(None, ArcBytes::from(b"a"), ArcBytes::from(b"1"))
+        tree.set(None, ArcBytes::from(b"a"), ArcBytes::from(b"1"))
             .unwrap();
-        tree.push(None, ArcBytes::from(b"a"), ArcBytes::from(b"2"))
+        tree.set(None, ArcBytes::from(b"a"), ArcBytes::from(b"2"))
             .unwrap();
 
         // Retrieve the sequences
@@ -2048,4 +2150,217 @@ mod tests {
         assert_eq!(sequences[1].1, b"1");
         assert_eq!(sequences[2].1, b"2");
     }
+
+    struct ExtendToPageBoundaryPlus(u64);
+
+    impl FileOp<()> for ExtendToPageBoundaryPlus {
+        #[allow(clippy::cast_possible_truncation)]
+        fn execute(self, file: &mut dyn File) {
+            let length = file.length().unwrap();
+            let bytes = vec![42_u8; (length - (length % PAGE_SIZE as u64) + self.0) as usize];
+            file.write_all(&bytes).unwrap();
+        }
+    }
+
+    #[test]
+    fn header_incompatible() {
+        let context = Context {
+            file_manager: MemoryFileManager::default(),
+            vault: None,
+            cache: None,
+        };
+        let temp_dir = crate::test_util::TestDirectory::new("header_incompatible");
+        std::fs::create_dir(&temp_dir).unwrap();
+        let file_path = temp_dir.join("tree");
+
+        // Write some data using unversioned
+        {
+            let state = State::default();
+            let mut tree =
+                TreeFile::<Unversioned, MemoryFile>::write(&file_path, state, &context, None)
+                    .unwrap();
+            tree.set(
+                None,
+                ArcBytes::from(b"test"),
+                ArcBytes::from(b"hello world"),
+            )
+            .unwrap();
+        }
+        // Try reading it as versioned.
+        let mut file = context.file_manager.append(&file_path).unwrap();
+        file.execute(ExtendToPageBoundaryPlus(3));
+        let state = State::default();
+        assert!(matches!(
+            TreeFile::<Versioned, MemoryFile>::write(&file_path, state, &context, None)
+                .unwrap_err()
+                .kind,
+            ErrorKind::DataIntegrity(_)
+        ));
+    }
+
+    #[test]
+    fn file_length_page_offset_plus_a_little() {
+        let context = Context {
+            file_manager: MemoryFileManager::default(),
+            vault: None,
+            cache: None,
+        };
+        let temp_dir = crate::test_util::TestDirectory::new("page-header-edge-cases");
+        std::fs::create_dir(&temp_dir).unwrap();
+        let file_path = temp_dir.join("tree");
+
+        // Write some data.
+        {
+            let state = State::default();
+            let mut tree =
+                TreeFile::<Unversioned, MemoryFile>::write(&file_path, state, &context, None)
+                    .unwrap();
+            tree.set(
+                None,
+                ArcBytes::from(b"test"),
+                ArcBytes::from(b"hello world"),
+            )
+            .unwrap();
+        }
+        // Test when the file is of a length that is less than 4 bytes longer than a multiple of a PAGE_SIZE.
+        let mut file = context.file_manager.append(&file_path).unwrap();
+        file.execute(ExtendToPageBoundaryPlus(3));
+        let state = State::default();
+        let mut tree =
+            TreeFile::<Unversioned, MemoryFile>::write(&file_path, state, &context, None).unwrap();
+
+        assert_eq!(tree.get(b"test", false).unwrap().unwrap(), b"hello world");
+    }
+
+    fn edit_keys<R: Root, M: FileManager>(label: &str, file_manager: M) {
+        let context = Context {
+            file_manager,
+            vault: None,
+            cache: None,
+        };
+        let temp_dir = crate::test_util::TestDirectory::new(format!("edit-keys-{}", label));
+        std::fs::create_dir(&temp_dir).unwrap();
+        let file_path = temp_dir.join("tree");
+
+        let mut tree =
+            TreeFile::<R, M::File>::write(&file_path, State::default(), &context, None).unwrap();
+        assert!(matches!(
+            tree.compare_and_swap(b"test", Some(b"won't match"), None, None)
+                .unwrap_err(),
+            CompareAndSwapError::Conflict(_)
+        ));
+        tree.compare_and_swap(b"test", None, Some(ArcBytes::from(b"first")), None)
+            .unwrap();
+        assert!(matches!(
+            tree.compare_and_swap(b"test", Some(b"won't match"), None, None)
+                .unwrap_err(),
+            CompareAndSwapError::Conflict(_)
+        ));
+        tree.compare_and_swap(
+            b"test",
+            Some(b"first"),
+            Some(ArcBytes::from(b"second")),
+            None,
+        )
+        .unwrap();
+
+        let stored = tree.replace(b"test", b"third", None).unwrap().unwrap();
+        assert_eq!(stored, b"second");
+
+        tree.compare_and_swap(b"test", Some(b"third"), None, None)
+            .unwrap();
+        assert!(tree.get(b"test", false).unwrap().is_none());
+    }
+
+    #[test]
+    fn std_edit_keys_versioned() {
+        edit_keys::<Versioned, _>("versioned", StdFileManager::default());
+    }
+
+    #[test]
+    fn std_edit_keys_unversioned() {
+        edit_keys::<Unversioned, _>("unversioned", StdFileManager::default());
+    }
+
+    #[test]
+    fn memory_edit_keys_versioned() {
+        edit_keys::<Versioned, _>("versioned", MemoryFileManager::default());
+    }
+
+    #[test]
+    fn memory_edit_keys_unversioned() {
+        edit_keys::<Unversioned, _>("unversioned", MemoryFileManager::default());
+    }
+
+    #[test]
+    fn any_edit_keys_versioned() {
+        edit_keys::<Versioned, _>("any-versioned", AnyFileManager::std());
+        edit_keys::<Versioned, _>("any-versioned", AnyFileManager::memory());
+    }
+
+    #[test]
+    fn any_edit_keys_unversioned() {
+        edit_keys::<Unversioned, _>("any-unversioned", AnyFileManager::std());
+        edit_keys::<Unversioned, _>("any-unversioned", AnyFileManager::memory());
+    }
+
+    fn first_last<R: Root, M: FileManager>(label: &str, file_manager: M) {
+        let context = Context {
+            file_manager,
+            vault: None,
+            cache: None,
+        };
+        let temp_dir = crate::test_util::TestDirectory::new(format!("first-last-{}", label));
+        std::fs::create_dir(&temp_dir).unwrap();
+        let file_path = temp_dir.join("tree");
+
+        let mut tree =
+            TreeFile::<R, M::File>::write(&file_path, State::default(), &context, None).unwrap();
+        tree.set(None, ArcBytes::from(b"a"), ArcBytes::from(b"first"))
+            .unwrap();
+        tree.set(None, ArcBytes::from(b"z"), ArcBytes::from(b"last"))
+            .unwrap();
+
+        assert_eq!(tree.first_key(false).unwrap().unwrap(), b"a");
+        let (key, value) = tree.first(false).unwrap().unwrap();
+        assert_eq!(key, b"a");
+        assert_eq!(value, b"first");
+
+        assert_eq!(tree.last_key(false).unwrap().unwrap(), b"z");
+        let (key, value) = tree.last(false).unwrap().unwrap();
+        assert_eq!(key, b"z");
+        assert_eq!(value, b"last");
+    }
+
+    #[test]
+    fn std_first_last_versioned() {
+        first_last::<Versioned, _>("versioned", StdFileManager::default());
+    }
+
+    #[test]
+    fn std_first_last_unversioned() {
+        first_last::<Unversioned, _>("unversioned", StdFileManager::default());
+    }
+
+    #[test]
+    fn memory_first_last_versioned() {
+        first_last::<Versioned, _>("versioned", MemoryFileManager::default());
+    }
+
+    #[test]
+    fn memory_first_last_unversioned() {
+        first_last::<Unversioned, _>("unversioned", MemoryFileManager::default());
+    }
+
+    #[test]
+    fn any_first_last_versioned() {
+        first_last::<Versioned, _>("any-versioned", AnyFileManager::std());
+        first_last::<Versioned, _>("any-versioned", AnyFileManager::memory());
+    }
+
+    #[test]
+    fn any_first_last_unversioned() {
+        first_last::<Unversioned, _>("any-unversioned", AnyFileManager::std());
+        first_last::<Unversioned, _>("any-unversioned", AnyFileManager::memory());
+    }
 }
